/* Factor - factorize numbers
   Copyright (C) 2006 Håvard Moen <post@haavard.name>

   Factor is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as published
   by the Free Software Foundation.

   Factor is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Factor; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */

import javax.microedition.midlet.MIDlet;
import javax.microedition.lcdui.*;

public class Factor extends MIDlet implements CommandListener {

  private Form myForm;
  private TextField input;
  private TextField inputMin;
  private TextField inputMax;
  private StringItem resultDisplay;
  private Command factorCommand = new Command("Factor number", Command.OK, 1);
  private Command findCommand = new Command("Find prime number", Command.OK, 1);
  private Command selectFactorCommand = new Command("Go to factor number", Command.SCREEN, 2);
  private Command selectFindCommand = new Command("Go to find prime number", Command.SCREEN, 2);
  private Command exitCommand = new Command("Exit", Command.EXIT, 1);

  private FactorThread factorThread = null;
  private FindThread findThread = null;

  public void startApp() {
    myForm = new Form("Factor");
    Display.getDisplay(this).setCurrent(myForm);
    myForm.setCommandListener(this);
    drawFactor();
  }

  private void drawFactor() {
    myForm.deleteAll();

    int size = Long.toString(Long.MAX_VALUE).length();
    input = new TextField("Number to factor", null, size, TextField.NUMERIC);
    myForm.append(input);

    resultDisplay = new StringItem("", "");
    myForm.append(resultDisplay);

    myForm.addCommand(factorCommand);
    myForm.addCommand(exitCommand);
    myForm.addCommand(selectFindCommand);
    myForm.removeCommand(selectFactorCommand);
    myForm.removeCommand(findCommand);
  }

  private void drawFind() {
    myForm.deleteAll();

    int size = Long.toString(Long.MAX_VALUE).length();
    inputMin = new TextField("Minimum", null, size, TextField.NUMERIC);
    inputMax = new TextField("Maximum", null, size, TextField.NUMERIC);
    myForm.append(inputMin);
    myForm.append(inputMax);

    resultDisplay = new StringItem("", "");
    myForm.append(resultDisplay);

    myForm.addCommand(findCommand);
    myForm.addCommand(exitCommand);
    myForm.addCommand(selectFactorCommand);
    myForm.removeCommand(selectFindCommand);
    myForm.removeCommand(factorCommand);
  }

  public void pauseApp() {
  }

  public void destroyApp(boolean unconditional) {
  }

  public void commandAction(Command c, Displayable d) {
    if (c == exitCommand) {
      destroyApp(false);
      notifyDestroyed();
    } else if (c == factorCommand) {
      if (input.size() == 0) {
        resultDisplay.setText("No number entered!");
      } else {
        resultDisplay.setText("");
        try {
          long i = Long.parseLong(input.getString());
          if (i < 0) {
            throw new NumberFormatException("The number must be positive");
          }
          resultDisplay.setText(Long.toString(i) + ":\n");
          input.setString("");
          factorThread = new FactorThread(i, resultDisplay);
          factorThread.start();
        } catch (NumberFormatException e) {
          resultDisplay.setText(e.getMessage());
        }
      }
    } else if (c == findCommand) {
      if (inputMin.size() == 0) {
        resultDisplay.setText("No minimum number entered!");
      } else if (inputMax.size() == 0) {
        resultDisplay.setText("No max number entered!");
      } else {
        resultDisplay.setText("");
        try {
          long min = Long.parseLong(inputMin.getString());
          long max = Long.parseLong(inputMax.getString());
          if (min < 0 || max < 0) {
            throw new NumberFormatException("The numbers must be positive");
          }
          if (max < min) {
            long temp = min;
            min = max;
            max = temp;
          }
          resultDisplay.setText("Prime numbers between " + Long.toString(min) + " and " + Long.toString(max) + ":\n");
          inputMin.setString("");
          inputMax.setString("");
          findThread = new FindThread(min, max, resultDisplay);
          findThread.start();
        } catch (NumberFormatException e) {
          resultDisplay.setText(e.getMessage());
        }
      }
    } else if (c == selectFactorCommand) {
      drawFactor();
    } else if (c == selectFindCommand) {
      drawFind();
    }
  }
}
